#!/bin/sh
# Licensed Materials - Property of IBM
#
# (C) Copyright IBM Corp. 2013 All Rights Reserved
#
# US Government Users Restricted Rights - Use, duplicate or
# disclosure restricted by GSA ADP Schedule Contract with
# IBM Corp.
# IBM_COPYRIGHT_END


usage(){
	echo "Usage of JRE update for Agent:"
	echo "updateAgentJRE.sh [root path of Agent]"
	echo "Example: updateAgentJRE.sh /opt/ibm/director/agent"
}

updateJRE(){
	timestamp=`date +%Y%m%d%H%M%S`
	echo "Updating JRE at $timestamp"
	oldJre="$jrePath.old.$timestamp"
	
	if [ ! -d "$tmpJre" ]; then
		echo "The new JRE is not ready in $tmpJre"
		return 1
	fi

	killOtherProcesses
	
	mv -f "$jrePath" "$oldJre"
	if [ "$?" -ne 0 ]; then
		echo "Failed to move current JRE."
		return 1
	else
		echo "Moved current JRE to $oldJre"
	fi
	
	mv -f "$tmpJre" "$jrePath"
	if [ "$?" -ne 0 ]; then
		echo "Failed to move new JRE from $tmpJre to $jrePath"
		if [ ! -d "$jrePath" ]; then
			mv -f "$oldJre" "$jrePath"
			echo "Reverted to original JRE via $oldJre"
		fi
		return 1
	else
		echo "Moved new JRE from $tmpJre to $jrePath successfully."
	fi

	setJREPermissions

	rm -rf "$oldJre"
	if [ "$?" -ne 0 ]; then
		echo "Failed to delete old JRE $oldJre at this time."
	else
		echo "Deleted old JRE $oldJre successfully."
	fi

	echo "Completed JRE update:"
	"$jrePath/bin/java" -version 2>> "$logFile"
}

killOtherProcesses(){
	echo "Finding all processes running agent JRE..."
	allProcesses=`ps -ef | grep "$jrePath/bin" | grep -v grep`
	if [ -n "$allProcesses" ]; then
		echo "-----------------------------------------------"
		echo "$allProcesses"
		echo "-----------------------------------------------"
	else
		echo "There is nothing running the agent JRE."
		return
	fi
	
	otherPIDs=`ps -ef | grep "$jrePath/bin" | grep -v grep | awk '{print $2}'`
	if [ -n "$otherPIDs" ]; then
		kill -9 $otherPIDs
		echo "Killed processes running the agent JRE:"
		echo "$otherPIDs"
		echo "-----------------------------------------------"
	else
		echo "There is no anything else running the agent JRE."
	fi
}

setJREPermissions(){
	chmod -R 755 "$jrePath"/bin
	if [ "$?" -ne 0 ]; then
		echo "Failed to set 755 permission for JRE bin files."
	fi
	chmod -R -f 755 "$jrePath"/plugin/desktop	
	chmod -R -f 644 "$jrePath"/lib "$jrePath"/docs "$jrePath"/properties
	chmod -R +X "$jrePath"
	chmod -f 644 "$jrePath"/plugin/desktop/*.png
	chmod -f 644 "$jrePath"/bin/*.properties
	chmod -f 644 "$jrePath"/copyright
	chmod -f 644 "$jrePath"/*.txt
	echo "Set JRE file permissions."
}

unalias -a

# To solve the ERROR: ld.so: object '/lib/libpam.so.0' from LD_PRELOAD cannot be preloaded: ignored.
# unset the environment variable LD_PRELOAD=/lib/libpam.so.0 which is set by parent launchlwi.sh script.
unset LD_PRELOAD

if [ $# == 0 ] || [ $# == 1 ]; then
	if [ $# == 0 ]; then
		scriptDir=$(cd "$(dirname "$0")"; pwd)
		rootPath="${scriptDir%/bin*}"
		if [ $scriptDir == $rootPath ]; then
			rootPath="${scriptDir%/conf/exescripts/prelaunch_once*}"
		fi
	else
		rootPath="$1"
	fi

	if [ ! -d "$rootPath" ]; then
		echo "The root path of Agent does not exist: $rootPath"
	else
		logDir="$rootPath/log"
		if [ ! -d "$logDir" ]; then
			mkdir "$logDir"
			echo "Created log folder: $logDir"
		fi
		jrePath="$rootPath/_jvm/jre"
		tmpJre="$rootPath/_jvm/jre.new"
		logFile="$rootPath/log/updateinstaller.log"
		updateJRE 2>&1 | tee -a "$logFile"
		echo "The update ended at `date +%Y%m%d%H%M%S`" | tee -a "$logFile"
	fi
else
	usage
fi